<?php

namespace App\Persistence\Eloquent;

use App\Persistence\Repositories\IPostRepository;
use App\Post;
use Illuminate\Contracts\Pagination\LengthAwarePaginator;
use Illuminate\Database\Eloquent\Builder;

class PostRepository extends Repository implements IPostRepository
{
    protected function getQueryBuilder(): Builder
    {
        return Post::query();
    }

    /**
     * @param int $pageSize
     * @param array $criteria
     * @param array|null $orderBy
     * @return LengthAwarePaginator
     */
    public function findPaginate($pageSize = 10, array $criteria = null, array $orderBy = null)
    {
        $query = $this->getQueryBuilder();
        if (!empty($criteria)) {
            $query->whereRowValues(array_keys($criteria), '=', array_values($criteria));
        }
        if (!empty($orderBy)) {
            foreach ($orderBy as $column => $direction) {
                $query->orderBy($column, $direction);
            }
        } else {
            $query->latest();
        }
        return $query->paginate($pageSize);
    }
}